﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=5.8
@EndOfDesignText@
'Class module

'http://www.dafaweek.com/HebCal/HebCalVb6.php

' This code demonstrates how to convert a Hebrew date into a
' Gregorian date. The code is written in VB6/VBA, but I purposely
' used very generic features so it would be easy to translate
' this into other languages. Also, I avoided using many
' optimization in order to make the logic clearer.

' These functions assume that all the current rules of the
' Hebrew calendar were always in existence (which is not true
' since the Hebrew calendar was not always fixed) and all the
' current rules of the Gregorian calendar were always in existence
' (which is not true).

' Here is a very brief description of the Hebrew calendar.
'
' The Hebrew calendar is a lunisolar calendar.  This means that
' the months are in sync with the moon and the years stay in sync
' with the sun.  A solar year is approximately 365.25 days.  A
' lunar month is approximately 29.5 days.  Twelve lunar months is
' approximately 354 days (12 * 29.5=354).  Thus, a lunar year of
' twelve months is 11.25 days shorter than the solar year. To make
' up for this shortfall, the Hebrew calendar adds a thirteenth
' month to seven years over a nineteen year period. Therefore, over
' a nineteen year period, the Hebrew calendar is approximately the
' same length as a nineteen year solar calendar.
'
' In order to understand this code, you must know the following
' terms:
'   Molad - new moon. Hebrew months start around the day of the
'           new moon
'   Chalakim - 1 / 1080 of an hour or 3 1/3 seconds
'   Tishrei - the first month of the Hebrew year (at least for
'             these calculations)
'   Rosh Hashanah - The Jewish new year which starts on Tishrei 1.
'
' The Hebrew calendar assumes the period of time between one new
' moon to the next is 29 days, 12 hours and 793 chalakim. The first
' molad after creation occurred on Monday, September, 7th -3760 at 5
' hours and 204 chalakim.  Technically, the Gregorian date would be
' in the year 3761 BCE because there was no year 0 in the Gregorian
' calendar, but we will use the year of -3760.

Sub Class_Globals
	Private fx As JFX
	Private Alpha As String
'	Private AlphaH() As String
End Sub


'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	DateTime.DateFormat = "MM/dd/yyyy"
	'1488 +
'	AlphaH = Array As String("Alef", "Bet", "Guimel", "Dalet", "He", "Vav", "Zayin", "Het", "Tet", "Yod", _
'	"Kaf_End", "Kaf", "Lamed", "Mem_End", "Mem", "Nun_End", "Nun", "Samekh", "Ayin", "Pe_End", "Pe", _
'	"Tsade_End", "Tsade", "Qoph", "Resh", "Shin", "Tav")
	Alpha = "ABGDHVZJTYcCLmMnNSIpPqQKRWX"
End Sub

' This function returns how many months there has been from the
' first Molad until the beginning of the year nYearH
Public Sub MonSinceFirstMolad(nYearH As Int) As Int
	Dim nMonSinceFirstMolad As Int

	' A shortcut to this function can simply be the following formula
	'   Return Int(((235 * nYearH) - 234) / 19)
	' This formula is found in Remy Landau's website and he
	' attributes it to Wolfgang Alexander Shochen. I will use a less
	' optimized function which I believe shows the underlying logic
	' better.

	' count how many months there has been in all years up to last
	' year. The months of this year hasn't happened yet.
	nYearH = nYearH - 1


	' In the 19 year cycle, there will always be 235 months. That
	' would be 19 years times 12 months plus 7 extra month for the
	' leap years. (19 * 12) + 7 = 235.

	' Get how many 19 year cycles there has been and multiply it by
	' 235
	nMonSinceFirstMolad = (nYearH / 19) 
	nMonSinceFirstMolad = nMonSinceFirstMolad * 235

	' Get the remaining years after the last complete 19 year cycle
	nYearH = nYearH Mod 19

	' Add 12 months for each of those years
	nMonSinceFirstMolad = nMonSinceFirstMolad + (12 * nYearH)

	' Add the extra months to account for the leap years
	If nYearH >= 17 Then
		nMonSinceFirstMolad = nMonSinceFirstMolad + 6
	Else If nYearH >= 14 Then
		nMonSinceFirstMolad = nMonSinceFirstMolad + 5
	Else If nYearH >= 11 Then
		nMonSinceFirstMolad = nMonSinceFirstMolad + 4
	Else If nYearH >= 8 Then
		nMonSinceFirstMolad = nMonSinceFirstMolad + 3
	Else If nYearH >= 6 Then
		nMonSinceFirstMolad = nMonSinceFirstMolad + 2
	Else If nYearH >= 3 Then
		nMonSinceFirstMolad = nMonSinceFirstMolad + 1
	End If
	Return  nMonSinceFirstMolad
End Sub

' This function returns if a given year is a leap year.
Public Sub IsLeapYear(nYearH As Int) As Boolean
	Dim nYearInCycle As Int
	Dim IsLeap As Boolean
	' Find out which year we are within the cycle.  The 19th year of
	' the cycle will return 0
	nYearInCycle = nYearH Mod 19
	IsLeap = (nYearInCycle = 3) Or _
	           (nYearInCycle = 6) Or _
	           (nYearInCycle = 8) Or _
	           (nYearInCycle = 11) Or _
	           (nYearInCycle = 14) Or _
	           (nYearInCycle = 17) Or _
	           (nYearInCycle = 0)
	Return IsLeap
End Sub

' This function figures out the Gregorian Date that corresponds to
' the first day of Tishrei, the first month of the Hebrew
' calendar, for a given Hebrew year.
Public Sub Tishrei1(nYearH As Int) As String
	Dim nMonthsSinceFirstMolad As Int
	Dim nChalakim As Int
	Dim nHours As Int
	Dim nDays As Int
	Dim nDayOfWeek As Int
	Dim dTishrei1 As Long

	' We want to calculate how many days, hours and chalakim it has
	' been from the time of 0 days, 0 hours and 0 chalakim to the
	' molad at the beginning of year nYearH.
	'
	' The period between one new moon to the next is 29 days, 12
	' hours and 793 chalakim. We must multiply that by the amount
	' of months that transpired since the first molad. Then we add
	' the time of the first molad (Monday, 5 hours and 204 chalakim)
	nMonthsSinceFirstMolad = MonSinceFirstMolad(nYearH)
	nChalakim = 793 * nMonthsSinceFirstMolad
	nChalakim = nChalakim + 204

	' carry the excess Chalakim over to the hours
	nHours = (nChalakim / 1080)
	nChalakim = nChalakim Mod 1080

	nHours = nHours + (nMonthsSinceFirstMolad * 12)
	nHours = nHours + 5

	' carry the excess hours over to the days
	nDays = (nHours / 24)
	nHours = nHours Mod 24

	nDays = nDays + (29 * nMonthsSinceFirstMolad)
	nDays = nDays + 2


	' figure out which day of the week the molad occurs.
	' Sunday = 1, Moday = 2 ..., Shabbos = 0
	nDayOfWeek = nDays Mod 7

	' In a perfect world, Rosh Hashanah would be on the day of the
	' molad. The Hebrew calendar makes four exceptions where we
	' push off Rosh Hashanah one or two days. This is done to
	' prevent three situation. Without explaining why, the three
	' situations are:
	'   1) We don't want Rosh Hashanah to come out on Sunday,
	'      Wednesday or Friday
	'   2) We don't want Rosh Hashanah to be on the day of the
	'      molad if the molad occurs after the beginning of 18th
	'      hour.
	'   3) We want to limit years to specific lengths.  For non-leap
	'      years, we limit it to either 353, 354 or 355 days.  For
	'      leap years, we limit it to either 383, 384 or 385 days.
	'      If setting Rosh Hashanah to the day of the molad will
	'      cause this year, or the previous year to fall outside
	'      these lengths, we push off Rosh Hashanah to get the year
	'      back to a valid length.
	' This code handles these exceptions.
	If Not (IsLeapYear(nYearH)) And _
		nDayOfWeek = 3 And _
		(nHours * 1080) + nChalakim >= _
		(9 * 1080) + 204 Then

		' This prevents the year from being 356 days. We have to push
		' Rosh Hashanah off two days because if we pushed it off only
		' one day, Rosh Hashanah would comes out on a Wednesday. Check
		' the Hebrew year 5745 for an example.
		nDayOfWeek = 5
		nDays = nDays + 2
	Else If IsLeapYear(nYearH - 1) And _
	     nDayOfWeek = 2 And _
	     (nHours * 1080) + nChalakim >= _
	     (15 * 1080) + 589 Then

		' This prevents the previous year from being 382 days. Check
		' the Hebrew Year 5766 for an example. If Rosh Hashanah was not
		' pushed off a day then 5765 would be 382 days
		nDayOfWeek = 3
		nDays = nDays + 1
	Else
		' see rule 2 above. Check the Hebrew year 5765 for an example
		If nHours >= 18 Then
			nDayOfWeek = nDayOfWeek + 1
			nDayOfWeek = nDayOfWeek Mod 7
			nDays = nDays + 1
		End If

		' see rule 1 above. Check the Hebrew year 5765 for an example
		If nDayOfWeek = 1 Or nDayOfWeek = 4 Or nDayOfWeek = 6 Then
			nDayOfWeek = nDayOfWeek + 1
			nDayOfWeek = nDayOfWeek Mod 7
			nDays = nDays + 1
		End If
	End If

	' Here we want to add nDays to creation
	'    dTishrie1 = creation + nDays
	' Unfortunately, VB doesn't handle negative years very well.
	' I therefore picked a Random date (1/1/1900) and figured out how
	' many days it is after the creation (2067025). Then I subtracted
	' 2067025 from nDays.
	nDays = nDays - 2067025
	dTishrei1 = DateTime.DateParse("1/1/1900")
	' 2067025 days after creation
	dTishrei1 = dTishrei1  + (DateTime.TicksPerDay * nDays)
	Return DateTime.Date(dTishrei1)
End Sub

' This function gets the length of a Hebrew year.
Public Sub LengthOfYear(nYearH As Int) As Int
	Dim dThisTishrei1 As String
	Dim dNextTishrei1 As String
	Dim diff As Long

	' subtract the date of this year from the date of next year
	dThisTishrei1 = Tishrei1(nYearH)
	dNextTishrei1 = Tishrei1(nYearH + 1)
	diff = (DateTime.DateParse(dNextTishrei1) - DateTime.DateParse(dThisTishrei1))/DateTime.TicksPerDay
	Return diff
End Sub

' This function converts a Hebrew date into the Gregorian date
' nYearH - is the Hebrew year
' nMonth - Tishrei=1
'          Heshvan=2
'          Kislev=3
'          Tevet=4
'          Shevat=5
'          Adar_I=6 (only valid on leap years)
'          Adar=7   (Adar_II for leap years)
'          Nissan=8
'          Iyar=9
'          Sivan=10
'          Tammuz=11
'          Av=12
'          Elul=13
Public Sub HebToGreg(nYearH As Int, nMonthH As Int, nDateH As String) As String
	Dim nLengthOfYear As Int
	Dim bLeap As Boolean
	Dim dGreg As String
	Dim nMonth As Int
	Dim nMonthLen As Int
	Dim bHaser As Boolean
	Dim bShalem As Boolean

	bLeap = IsLeapYear(nYearH)
	nLengthOfYear = LengthOfYear(nYearH)

	' The regular length of a non-leap year is 354 days.
	' The regular length of a leap year is 384 days.
	' On regular years, the length of the months are as follows
	'   Tishrei (1)   30
	'   Heshvan(2)   29
	'   Kislev  (3)   30
	'   Tevet  (4)   29
	'   Shevat  (5)   30
	'   Adar_I  (6)   30     (only valid on leap years)
	'   Adar    (7)   29     (Adar_II for leap years)
	'   Nissan   (8)   30
	'   Iyar    (9)   29
	'   Sivan   (10)  30
	'   Tammuz   (11)  29
	'   Av      (12)  30
	'   Elul    (13)  29
	' If the year is shorter by one less day, it is called a haser
	' year. Kislev on a haser year has 29 days. If the year is longer
	' by one day, it is called a shalem year. Heshvan on a shalem
	' year is 30 days.
	bHaser = nLengthOfYear = 353 Or nLengthOfYear = 383
	bShalem = nLengthOfYear = 355 Or nLengthOfYear = 385

	' get the date for Tishrei 1
	dGreg = Tishrei1(nYearH)

	' Now count up days within the year
	For nMonth = 1 To nMonthH - 1
		Select Case nMonth
			Case 1, 5, 8, 10, 12 ' 30 day months
		    	nMonthLen = 30
			Case 4, 7, 9, 11, 13 ' 29 day months
		    	nMonthLen = 29
		  	Case 6 ' There is only an Adar_I on a leap years
		  		If bLeap Then 
					nMonthLen = 30
				Else
					nMonthLen = 0
				End If
		  	Case 2 ' Heshvan, see note above
		  		If bShalem Then 
					nMonthLen = 30
				Else
					nMonthLen = 29
				End If	  	
		  	Case 3 ' Kislev, see note above
		  		If bHaser Then 
					nMonthLen = 29
				Else
					nMonthLen = 30
				End If
		End Select
		dGreg = DateTime.Date(DateTime.DateParse(dGreg) + DateTime.TicksPerDay * nMonthLen)
	Next
	dGreg = DateTime.Date(DateTime.DateParse(dGreg) + DateTime.TicksPerDay * (nDateH - 1))
	Return dGreg
End Sub

' This function converts a Gregorian date into the Hebrew date.  The
' function returns the hebrew month as a string in the format MM/DD/YYYY.
' Also, the parameters nYearH, nMonthH and hDateH, which are sent by
' reference, will get set the Hebrew year, month and date. See function
' HebToGreg() for the definition of the month numbers.
Public Sub GregToHeb(dGreg As String) As String
	Dim nOneMolad As Double
	Dim nAvrgYear As Double
	Dim nDays As Int
	Dim dTishrei1 As String
	Dim nLengthOfYear As Int
	Dim bLeap As Boolean
	Dim bHaser As Boolean
	Dim bShalem As Boolean
	Dim nMonthLen As Int
	Dim bWhile As Boolean = True
	Dim nYearH, nMonthH, nDateH As Int

	' The basic algorythm to get Hebrew date for the Gregorian date dGreg.
	' 1) Find out how many days dGreg is after creation.
	' 2) Based on those days, estimate the Hebrew year
	' 3) Now that we a good estimate of the Hebrew year, use brute force to
	'    find the Gregorian date for Tishrei 1 prior to or equal to dGreg
	' 4) Add to Tishrei 1 the amount of days dGreg is after Tishrei 1

	' Figure out how many days are in a month.
	' 29 days + 12 hours + 793 chalakim
	nOneMolad = 29 + (12 / 24) + (793 / (1080 * 24))

	' Figure out the average length of a year. The hebrew year has exactly
	' 235 months over 19 years.
	nAvrgYear = nOneMolad * (235 / 19)

	' Get how many days dGreg is after creation. See note as to why I
	' use 1/1/1900 and add 2067025
	nDays = Round((DateTime.DateParse(dGreg) - DateTime.DateParse("1/1/1900")) / DateTime.TicksPerDay)
	nDays = nDays + 2067025 ' 2067025 days after creation

	' Guess the Hebrew year. This should be a pretty accurate guess.
	nYearH = nDays / nAvrgYear + 1

	' Use brute force to find the exact year nYearH. It is the Tishrei 1 in
	' the year <= dGreg.
	dTishrei1 = Tishrei1(nYearH)
	If dTishrei1 = dGreg Then

		' If we got lucky and landed on the exact date, we can stop here
		nMonthH = 1
		nDateH = 1
	Else

		' Here is the brute force.  Either count up or count down nYearH
		' until Tishrei 1 is <= dGreg.
		If DateTime.DateParse(dTishrei1) < DateTime.DateParse(dGreg) Then
			' If Tishrei 1, nYearH is less than dGreg, count nYearH up.
			Do While DateTime.DateParse(Tishrei1(nYearH + 1)) <= DateTime.DateParse(dGreg)
				nYearH = nYearH + 1
			Loop
		Else

	  		' If Tishrei 1, nYearH is greater than dGreg, count nYearH down.
	  	nYearH = nYearH - 1
	  	Do While DateTime.DateParse(Tishrei1(nYearH)) > DateTime.DateParse(dGreg)
	    	nYearH = nYearH - 1
	  	Loop
		End If

		' Subtract Tishrei 1, nYearH from dGreg. That should leave us with
		' how many days we have to add to Tishrei 1
		nDays = Round((DateTime.DateParse(dGreg) - DateTime.DateParse(Tishrei1(nYearH))) / DateTime.TicksPerDay)

		' Find out what type of year it is so that we know the length of the
		' months
		nLengthOfYear = LengthOfYear(nYearH)
		bHaser = nLengthOfYear = 353 Or nLengthOfYear = 383
		bShalem = nLengthOfYear = 355 Or nLengthOfYear = 385
		bLeap = IsLeapYear(nYearH)

		' Add nDays to Tishrei 1.
		nMonthH = 1
		Do While bWhile
	  		Select Case nMonthH
	    		Case 1, 5, 8, 10, 12 ' 30 day months
	      			nMonthLen = 30
	    		Case 4, 7, 9, 11, 13 ' 29 day months
	      			nMonthLen = 29
	    		Case 6 ' Adar_I (6) will be skipped on non-leap years
	      			nMonthLen = 30
	    		Case 2 ' Heshvan, see note above
					If bShalem Then
						nMonthLen = 30
					Else
						nMonthLen = 29
					End If
	    		Case 3 ' Kislev, see note above
					If bHaser Then
						nMonthLen = 29
					Else
						nMonthLen = 30
					End If
	  		End Select
	  		If nDays >= nMonthLen Then
	    		bWhile = True
	    		If bLeap Or nMonthH <> 5 Then
	      			nMonthH = nMonthH + 1
	    		Else
	      			' We can skip Adar_I (6) if its not a leap year
	      			nMonthH = nMonthH + 2
	    		End If
	    		nDays = nDays - nMonthLen
	  		Else
	    		bWhile = False
	  		End If
		Loop 'While bWhile

		' Add the remaining days to Date
		nDateH = nDays + 1
	End If
	Return FormatDateH(nYearH, nMonthH, nDateH)
End Sub

Public Sub FormatDateH(nYearH As Int, nMonthH As Int, nDateH As Int) As String
	Dim sMonth As String
  	Select Case nMonthH
    	Case 1
      		sMonth = "Tishrei"
    	Case 2
      		sMonth = "Heshvan"
    	Case 3
      		sMonth = "Kislev"
    	Case 4
      		sMonth = "Tevet"
    	Case 5
      		sMonth = "Shevat"
    	Case 6
      		sMonth = "Adar_I"
    	Case 7
			If IsLeapYear(nYearH) Then
				sMonth = "Adar_II"
			Else
				sMonth = "Adar"
			End If
    	Case 8
      		sMonth = "Nissan"
    	Case 9
      		sMonth = "Iyar"
    	Case 10
      		sMonth = "Sivan"
    	Case 11
      		sMonth = "Tammuz"
    	Case 12
      		sMonth = "Av"
    	Case 13
      		sMonth = "Elul"
	End Select
	Return sMonth & " " & nDateH & " " & nYearH
End Sub

Public Sub MonthHtoNum(nMonthH As String) As Short
	Dim sMonth As Short
  	Select Case nMonthH.ToLowerCase
    	Case "tishrei"
			sMonth = 1
    	Case "heshvan"
			sMonth = 2
    	Case "kislev"
			sMonth = 3
    	Case "tevet"
			sMonth = 4
    	Case "shevat"
			sMonth = 5
    	Case "adar_i"
			sMonth = 6
    	Case "adar_ii", "adar"
			sMonth = 7
    	Case "nissan"
			sMonth = 8
    	Case "iyar"
			sMonth = 9
    	Case "sivan"
			sMonth = 10
    	Case "tammuz"
			sMonth = 11
    	Case "av"
			sMonth = 12
    	Case "elul"
			sMonth = 13 
	End Select
	Return sMonth 
End Sub

Public Sub TodayHeb() As String
	Return GregToHeb(DateTime.Date(DateTime.Now))
End Sub

'Some helper hebrew functions

Private Sub HebrewNum(n As Int) As String
	Dim nX As Short
	Dim sHundred As String
	Dim sTen As String
	Dim sOne As String

	n = n Mod 1000
	nX = n - (n Mod 100)
	Select Case nX
		Case 900
			sHundred = ToHebSt("XXR") '("Tav") & ("Tav") & ("Resh")
		Case 800
			sHundred =  ToHebSt("XXK") '("Tav") & ("Tav") & ("Qoph")
		Case 700
			sHundred =  ToHebSt("XW") '("Tav") & ("Shin")
		Case 600
			sHundred =  ToHebSt("XR") '("Tav") & ("Resh")
		Case 500
			sHundred =  ToHebSt("XK") '("Tav") & ("Qoph")
		Case 400
			sHundred =  ToHebSt("X") '("Tav")
		Case 300
			sHundred =  ToHebSt("W") '("Shin")
		Case 200
			sHundred =  ToHebSt("R") '("Resh")
		Case 100
			sHundred =  ToHebSt("K") '("Qoph")
 	End Select
  	n = n - nX
  	If n = 15 Then
    	sTen = ToHebSt("T") '("Tet")
    	sOne = ToHebSt("V") '("Vav")
  	Else If n = 16 Then
    	sTen = ToHebSt("T") '("Tet")
    	sOne = ToHebSt("Z") '("Zayin")
  	Else
    	nX = n - (n Mod 10)
    	Select Case nX
      		Case 90
        		sTen = ToHebSt("Q") '("Tsade")
      		Case 80
        		sTen = ToHebSt("P") '("Pe")
      		Case 70
        		sTen = ToHebSt("I") '("Ayin")
      		Case 60
        		sTen = ToHebSt("S") '("Samekh")
      		Case 50
        		sTen = ToHebSt("N") '("Nun")
      		Case 40
        		sTen =ToHebSt("M") ' ("Mem")
      		Case 30
        		sTen = ToHebSt("L") '("Lamed")
      		Case 20
        		sTen = ToHebSt("C") '("Kaf")
      		Case 10
        		sTen = ToHebSt("Y") '("Yod")
    	End Select
    	nX = n - nX
    	Select Case nX
      		Case 9
        		sOne = ToHebSt("T") '("Tet")
      		Case 8
        		sOne = ToHebSt("J") '("Het")
      		Case 7
        		sOne = ToHebSt("Z") '("Zayin")
      		Case 6
        		sOne = ToHebSt("V") '("Vav")
      		Case 5
        		sOne = ToHebSt("H") '("He")
      		Case 4
        		sOne = ToHebSt("D") '("Dalet")
      		Case 3
        		sOne = ToHebSt("G") '("Guimel")
      		Case 2
        		sOne = ToHebSt("B") '("Bet")
      		Case 1
        		sOne = ToHebSt("A") '("Alef")
    	End Select
 	End If
 	Return sHundred & sTen & sOne
End Sub

Public Sub FormatDateHInHeb(nYearH As Int, nMonthH As Int, nDateH As Int) As String
	Dim sMonth As String
	Select Case nMonthH
		Case 1
			sMonth = ToHebSt("XWRY")
		Case 2
			sMonth = ToHebSt("JWVn")
		Case 3
			sMonth = ToHebSt("CSLV")
		Case 4
			sMonth = ToHebSt("TBX") 
		Case 5
			sMonth = ToHebSt("WBT") 
		Case 6
			sMonth = ToHebSt("ADR A")
		Case 7
			If IsLeapYear(nYearH) Then
				sMonth = ToHebSt("ADR B")
			Else
		        sMonth = ToHebSt("ADR") 
			End If
		Case 8
			sMonth = ToHebSt("NYSn") 
		Case 9
			sMonth = ToHebSt("AYYR") 
		Case 10
			sMonth = ToHebSt("SYVn") 
		Case 11
			sMonth = ToHebSt("XMVZ")
		Case 12
			sMonth = ToHebSt("AB") 
		Case 13
			sMonth = ToHebSt("ALVL")
	End Select
	Return  sMonth & " " & HebrewNum(nDateH) & " " & nYearH
	' Return  sMonth & " " & HebrewNum(nDateH) & " " & HebrewNum(nYearH)
End Sub

Public Sub ToHebSt(sHebSt As String) As String
	Dim pos As Short
	Dim sCh As String
	Dim sHebCh As String
	For i = 0 To sHebSt.Length - 1
		sCh = sHebSt.SubString2(i, i + 1)
		pos = Alpha.IndexOf(sCh)
		If pos < 0  Then
			sHebCh = sHebCh & sCh
		Else
			sHebCh = sHebCh & Chr(pos + 1488)		
		End If
	Next
	Return sHebCh
End Sub

'dg = date 
Public Sub GetParshaX(dg As String, nType As Short, bNoPRWX As Boolean) As String
	Dim nParshaNumber As Short
	Dim dTargetShabbos As String
	Dim dWorkingShabbos As String
	Dim nYearH As Int
	Dim nMonthH As Int
	Dim nDateH As Int
	Dim bIsYomTov As Boolean
	Dim bDoubleParsha As Boolean
	Dim bDone As Boolean
	Dim sParsha As String
	' Dim aParsha(64) As String
	nType = Min(Max(nType, 1),2)
	
	dTargetShabbos = ShabbosOfDate(dg)
	dWorkingShabbos = InitWorkingShabbos(dg)
	nParshaNumber = Round((DateTime.DateParse(dTargetShabbos) - DateTime.DateParse(dWorkingShabbos)) / (7 * DateTime.TicksPerDay))
	If nParshaNumber < 22 Then
	    sParsha = gParshaH(nParshaNumber, nType)
	Else
		Dim dtTk As Long = DateTime.DateParse(dWorkingShabbos)
		dWorkingShabbos = DateTime.Date(dtTk + (7* 22 * DateTime.TicksPerDay))
	    nParshaNumber = 22
		Do While Not(bDone)
			Dim dh As String = GregToHeb(dWorkingShabbos)
			Dim dtTmp() As String  = Regex.Split(" ", dh)
			nDateH  = dtTmp(1)
			nMonthH  = MonthHtoNum(dtTmp(0))
			nYearH  = dtTmp(2)
	      	bIsYomTov = IsYomTov(nYearH, nMonthH, nDateH)
			bDoubleParsha = False
			Select Case nParshaNumber
	        	Case 22
					If DaysBeforePesach(dWorkingShabbos, nYearH) < 21 Then
	            		bDoubleParsha = True
	          		End If
	        	Case 27, 29
	  				If Not(IsLeapYear(nYearH)) Then
						bDoubleParsha = True
					End If
				Case 32
					If IsIsrael Then
						' Inside Israel: Combine if Passover does not start on Shabbat AND NOT a leap year.
						' The Passover exception only occurs in a 354-day year in which Rosh HaShanah starts on Thursda
	          		Else
						If Not(IsLeapYear(nYearH)) Then
							bDoubleParsha = True
						End If
					End If
				Case 39
					If Not(IsIsrael) Then
						' if Pesach falls out on thursday
						Dim dgtk As Long = DateTime.DateParse(HebToGreg(nYearH, 8, 15))
						If DateTime.GetDayOfWeek(dgtk) = 5 Then 	'Thursday 
							bDoubleParsha = True
						End If
					End If
	        	Case 42
          			If DaysBeforeTishaBav(dWorkingShabbos, nYearH) < 14 Then
							bDoubleParsha = True
					End If
	        	Case 51
					Dim dgtk As Long = DateTime.DateParse(HebToGreg(nYearH + 1, 1, 1))
					Dim dgtk1 As Long = DateTime.DateParse(HebToGreg(nYearH + 1, 1, 10))
					If DateTime.GetDayOfWeek(dgtk) = 7 Or DateTime.GetDayOfWeek(dgtk1) = 7 Then 	'Saturday 
						bDoubleParsha = True
					End If
			End Select
			If dTargetShabbos = dWorkingShabbos Then
				If bIsYomTov Then
					If nType = 1 Then
						sParsha = TheYomTov(nYearH, nMonthH, nDateH, dWorkingShabbos)
					Else
						sParsha = TheYomTovHeb(nYearH, nMonthH, nDateH, dWorkingShabbos)
					End If
					bDone = True
				Else
          			sParsha = gParshaH(nParshaNumber, nType)
					If bDoubleParsha Then
						sParsha = sParsha & "-" & gParshaH(nParshaNumber + 1, nType)
					End If
					bDone = True
				End If
			End If
			If Not(bDone) Then
				If nParshaNumber > 64 Then
					sParsha = "Error" ' debug
					bDone = True
				Else
					If Not(bIsYomTov) Then
						If bDoubleParsha Then
							nParshaNumber = nParshaNumber + 2
						Else
							nParshaNumber = nParshaNumber + 1
						End If
					End If
					Dim dtTk As Long = DateTime.DateParse(dWorkingShabbos) + 7 * DateTime.TicksPerDay
					dWorkingShabbos = DateTime.Date(dtTk)
	        	End If
	      	End If
		Loop
	End If
	If Not(bIsYomTov) And sParsha.Length < 9 And Not(bNoPRWX) And nType = 2 Then
		sParsha = ToHebSt("PRWX") & " " & sParsha
	End If
	'Log(nParshaNumber)
	Return sParsha
End Sub


public Sub gParshaH(nNum As Short, nType As Short) As String
	If nNum > 64 Then Return "Error"
	Dim pos1, pos2 As Short
	pos2 = -1
	If nType = 1 Then
		Dim gParsha As String = "Bereishit,Noach,Lech Lecha,Vayeira,Chayei Sarah,Toledot,Vayeitzei,Vayishlach,Vayeishev" & _
		",Mikeitz,Vayigash,Vayechi,Shemot,Va'eira,Bo,Beshalach,Yitro,Mishpatim,Terumah,Titzaveh,Ki Tisa,Vayakhel,Pekudei" & _
		",Vayikra,Tzav,Shemini,Tazria,Metzora,Acharei Mot,Kedoshim,Emor,Behar,Bechukotai,Bamidbar,Nasso,Beha'alotcha,Shelach" & _
		",Korach,Chukat,Balak,Pinchas,Mattot,Masei,Devarim,Ve'etchanan,Eikev,Re'eh,Shoftim,Ki Teitzei,Ki Tavo,Nitzavim,Vayeilech" & _
		",Haazinu,V'zot HaBrachah,Rosh Hashana,Yom Kippur,Succos,Chol Hamoed Succos,Hoshana Raba,Shmini Atzeres" & _
		",Simchas Torah,Rosh Chodesh,Chanukah,Asara B'Tevet,"
	Else
		Dim gParsha As String = "BRAWYX,NJ,Lc Lc,VYRA,JYY WRH,XVLDX,VYQA,VYWLJ,VYWB,MKq,VYGW,VYJY,WMVX,VARA,BA" & _
		",BWLJ,YXRV,MWPTYm,XRVMH,XQVH,CY XWA,VYKHL,PKVDY,VYKRA,QV,WMYNY,XZRYI,MQVRI,AJRY MVX,KDWYm,AMVR,BHR,BJKXY" & _
		",BMDBR,NWA,BHILXc,WLJ Lc,KRJ,JKX,BLK,PYNJS,MTVX,MSIY,DBRYm,VAXJNn,IKB,RAH,WVPTYm,CY XQA,CY XBVA,NQBYm" & _
		",VYLc,HAZYNV,VZAX HBRCH,RAW HWNH,YVm CYPVR,SVCVX,JVH M SVCVX,HVWINA RBA,WMYNY IQRX,WMJX XVRH,RAW JVDW,JNVCH,IWRH BTBX,"
	End If
	For i = 0 To  nNum -1
		pos1 = pos2 + 1
		pos2 = gParsha.IndexOf2(",", pos1)
	Next
	If nType = 1 Then
		Return gParsha.SubString2(pos1,pos2)
	Else
		Return ToHebSt(gParsha.SubString2(pos1,pos2))
	End If
End Sub

Private Sub InitWorkingShabbos(dg As String) As String
 	dg = ShabbosOfDate(dg)
	dg = PreviousSimchasTorah(dg)
 	dg = ShabbosOfDate(dg)
	Dim dgTk As Long = DateTime.DateParse(dg) -7 * DateTime.TicksPerDay 
	dg = DateTime.Date(dgTk)
	Return dg
 	'InitWorkingShabbos = ShabbosOfDate(PreviousSimchasTorah(dg)) - 7
End Sub

Private Sub PreviousSimchasTorah(dg As String) As String
	Dim nYearH As Int
	Dim nMonthH As Int
	Dim nDateH As Int
	Dim nSTDateH As Int
	
	Dim dh As String = GregToHeb(dg)
	Dim dtTmp() As String  = Regex.Split(" ", dh)
	nDateH  = dtTmp(1)
	nMonthH  = MonthHtoNum(dtTmp(0))
	nYearH  = dtTmp(2)
	If IsIsrael Then
		nSTDateH = 22
	Else
		nSTDateH = 23
	End If
	If nDateH < nSTDateH And nMonthH = 1 Then
		nYearH = nYearH - 1
	End If
	Return HebToGreg(nYearH, 1, nSTDateH)
End Sub

Private Sub IsYomTov(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
 	Return 	IsRoshHashana(nYearH, nMonthH, nDateH) Or _
 			IsYomKippur(nYearH, nMonthH, nDateH) Or _
       		IsSuccos(nYearH, nMonthH, nDateH) Or _
            IsPesach(nYearH, nMonthH, nDateH) Or _
            IsShavuos(nYearH, nMonthH, nDateH)
End Sub

Private Sub IsRoshHashana(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	Return	nMonthH = 1 And _
            nDateH >= 1 And _
            nDateH <= 2
End Sub

Private Sub IsShavuos(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	Return IsYomTovX(nYearH, nMonthH, nDateH, 10, 6, 1)
End Sub

Private Sub IsYomKippur(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
  Return	nMonthH = 1 And _
            nDateH = 10
End Sub

Private Sub IsPesach(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	Return IsYomTovX(nYearH, nMonthH, nDateH, 8, 15, 7)
End Sub

Private Sub IsSuccos(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	Return IsYomTovX(nYearH, nMonthH, nDateH, 1, 15, 8)
End Sub

Private Sub IsSuccosYT(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	Return IsYomTovX(nYearH, nMonthH, nDateH, 1, 15, 1)
End Sub

Private Sub IsSuccosCH(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	If IsIsrael Then
		Return IsYomTovX(nYearH, nMonthH, nDateH, 1, 16, 4)
	Else
		Return IsYomTovX(nYearH, nMonthH, nDateH, 1, 17, 3)
	End If
End Sub

Private Sub IsYomTovX(nYearH As Int, nMonthH As Int, nDateH As Int, nYTMonth As Int, nYTDate As Int, nYTLen As Int) As Boolean
	Dim nEOYomTov As Int
	nEOYomTov = nYTDate + nYTLen - 1
	If Not(IsIsrael) Then
		nEOYomTov = nEOYomTov + 1
	End If
	Return	nMonthH = nYTMonth And _
	     	nDateH >= nYTDate And _
	        nDateH <= nEOYomTov
End Sub

Private Sub ShabbosOfDate(dg As String) As String
	Dim dgTk As Long
	dgTk = DateTime.DateParse(dg)
	Dim WkDay As Int = DateTime.GetDayOfWeek(dgTk)
  	WkDay = 7 - WkDay
	Return  DateTime.Date(dgTk + WkDay * DateTime.TicksPerDay)
End Sub

Private Sub DaysBeforePesach(dWorkingShabbos As String, nYearH As Int) As Short
  Dim dErevPesach As String
  dErevPesach = HebToGreg(nYearH, 8, 14)
  Return (DateTime.DateParse(dErevPesach) - DateTime.DateParse(dWorkingShabbos)) / DateTime.TicksPerDay
End Sub

Private Sub IsHoshanaRaba(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	Return nMonthH = 1 And nDateH = 21
End Sub

Private Sub IsShminiAtzeres(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	Return nMonthH = 1 And nDateH = 22
End Sub

Private Sub IsSimchasTorah(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	Dim Israel As Short
	If IsIsrael Then
		Israel = 22
	Else
		Israel = 23	
	End If
	Return nMonthH = 1 And nDateH = Israel
End Sub

Private Sub IsChanukah(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	Dim nLengthOfYear As Short
	Dim bHaser As Boolean
	Dim Chanukah As Boolean
	If nMonthH = 3 And nDateH >= 25 Then
		Chanukah = True
	Else If nMonthH = 4 Then
		nLengthOfYear = LengthOfYear(nYearH)
		bHaser = nLengthOfYear = 353 Or nLengthOfYear = 383
		If bHaser Then 
			Chanukah = nDateH <= 3
		Else
			Chanukah = nDateH <= 2
		End If
	Else
		Chanukah = False
	End If
	Return Chanukah
End Sub

Private Sub IsRoshChodesh(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
  Return nDateH = 1 Or nDateH = 30
End Sub

Private Sub IsPesachYT(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
  	Return IsYomTovX(nYearH, nMonthH, nDateH, 8, 15, 1) Or _
           IsYomTovX(nYearH, nMonthH, nDateH, 8, 21, 1)
End Sub

Private Sub IsPesachCH(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	If IsIsrael Then
		Return IsYomTovX(nYearH, nMonthH, nDateH, 8, 16, 4)
	Else
		Return IsYomTovX(nYearH, nMonthH, nDateH, 8, 17, 3)
	End If
End Sub

Private Sub Is10Teves(nYearH As Int, nMonthH As Int, nDateH As Int, dDate As String) As Boolean
	Return IsFastDay(nYearH, nMonthH, nDateH, dDate, 4, 10)
End Sub

Private Sub IsFastDay(nYearH As Int, nMonthH As Int, nDateH As Int, dDate As String, nMonthHX As Int, nDateHX As Int) As Boolean
	If nMonthH = nMonthHX Then
		If nDateH = nDateHX Then
			Return DateTime.GetDayOfWeek(DateTime.DateParse(dDate)) <> 7	'Saturday
		Else If nDateH = nDateHX + 1 Then
		  	Dim dtTk As Long = DateTime.DateParse(dDate) - 1 * DateTime.TicksPerDay
			Return  DateTime.GetDayOfWeek(dtTk) = 7 		'Saturday
		End If
	End If
	Return True
End Sub

Private Sub Is15Shevat(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	Return nMonthH = 5 And nDateH = 15
End Sub

Private Sub IsPurimKatan(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	Return nMonthH = 6 And nDateH = 14
End Sub

Private Sub IsShushanPurimKatan(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	Return nMonthH = 6 And nDateH = 15
End Sub

Private Sub IsPurim(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	Return nMonthH = 7 And nDateH = 14
End Sub

Private Sub IsShushanPurim(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	Return nMonthH = 7 And nDateH = 15
End Sub

Private Sub IsLagBOmer(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	Return nMonthH = 9 And nDateH = 18
End Sub

Private Sub Is17Tamuz(nYearH As Int, nMonthH As Int, nDateH As Int, dDate As String) As Boolean
  Return IsFastDay(nYearH, nMonthH, nDateH, dDate, 11, 17)
End Sub

Private Sub Is9BAv(nYearH As Int, nMonthH As Int, nDateH As Int, dDate As String) As Boolean
	Return IsFastDay(nYearH, nMonthH, nDateH, dDate, 12, 9)
End Sub

Private Sub Is15BAv(nYearH As Int, nMonthH As Int, nDateH As Int) As Boolean
	Return nMonthH = 12 And nDateH = 15
End Sub

Private Sub DaysBeforeTishaBav(dWorkingShabbos As String, nYearH As Int ) As Int
  Dim dTishaBav As String
  dTishaBav = HebToGreg(nYearH, 12, 9)
  Return (DateTime.DateParse(dTishaBav) - DateTime.DateParse(dWorkingShabbos)) / DateTime.TicksPerDay
End Sub

private Sub TheYomTov(nYearH As Int, nMonthH As Int, nDateH As Int, dDate As String) As String
	If IsRoshHashana(nYearH, nMonthH, nDateH) Then
		Return "Rosh Hashana"
	Else If IsYomKippur(nYearH, nMonthH, nDateH) Then
		Return "Yom Kippur"
	Else If IsSuccosYT(nYearH, nMonthH, nDateH) Then
		Return "Succos"
	Else If IsSuccosCH(nYearH, nMonthH, nDateH) Then
		Return "Chol Hamoed Succos"
	Else If IsHoshanaRaba(nYearH, nMonthH, nDateH) Then
		Return "Hoshana Raba"
	Else If IsShminiAtzeres(nYearH, nMonthH, nDateH) Then
		Return "Shmini Atzeres"
	Else If IsSimchasTorah(nYearH, nMonthH, nDateH) Then
		Return "Simchas Torah"
	Else If IsChanukah(nYearH, nMonthH, nDateH) And IsRoshChodesh(nYearH, nMonthH, nDateH) Then
		Return "Chanukah/R""Ch"
	Else If IsChanukah(nYearH, nMonthH, nDateH) Then
		Return "Chanukah"
	Else If IsRoshChodesh(nYearH, nMonthH, nDateH) Then
		Return "Rosh Chodesh"
	Else If IsPesachYT(nYearH, nMonthH, nDateH) Then
		Return "Pesach"
	Else If IsPesachCH(nYearH, nMonthH, nDateH) Then
		Return "Chol Hamoed Pesach"
	Else If IsShavuos(nYearH, nMonthH, nDateH) Then
		Return "Shevuos"
	Else
		Return ""
	End If
End Sub

Private Sub TheYomTovHeb(nYearH As Int, nMonthH As Int, nDateH As Int, dDate As String) As String
	If IsRoshHashana(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("RAW HWNH")
	Else If IsYomKippur(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("YVm CYPVR")
	Else if IsSuccosYT(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("SVCVX")
	Else if IsSuccosCH(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("JVH""M SVCVX")
	Else if IsHoshanaRaba(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("HVWINA RBA")
	Else if IsShminiAtzeres(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("WMYNY IQRX")
	Else if IsSimchasTorah(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("WMJX XVRH")
	Else if IsChanukah(nYearH, nMonthH, nDateH) And IsRoshChodesh(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("JNVCH / R""J")
	Else if IsChanukah(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("JNVCH")
	Else if IsRoshChodesh(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("RAW JVDW")
	Else if Is10Teves(nYearH, nMonthH, nDateH, dDate) Then
		Return ToHebSt("IWRH BTBX")
	Else if Is15Shevat(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("TV BWBT")
	Else if IsPurimKatan(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("PVRYm KTn")
	Else if IsShushanPurimKatan(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("W""P KTn")
	Else if IsPurim(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("PVRYm")
	Else if IsShushanPurim(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("WVWn PVRYm")
	Else if IsPesachYT(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("PSJ")
	Else if IsPesachCH(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("JVH""M PSJ")
	Else if IsLagBOmer(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("L""G BIVMR")
	Else if IsShavuos(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("WBVIVX")
	Else if Is17Tamuz(nYearH, nMonthH, nDateH, dDate) Then
		Return ToHebSt("Y""Z BXMVZ")
	Else if Is9BAv(nYearH, nMonthH, nDateH, dDate) Then
		Return ToHebSt("T' BAB")
	Else if Is15BAv(nYearH, nMonthH, nDateH) Then
		Return ToHebSt("T""V BAB")
	Else
		Return ""
	End If
End Sub

Private Sub IsIsrael() As Boolean
  Return False
End Sub